-- Auteur : Olivier Payrastre
-- Date : 29/10/2019
-- Version : 3.01

-- Langage : LUA/Domoticz
-- Mode : Time

-- Nom du script : script_time_gst_piscine.lua

-- Topic officiel : https://easydomoticz.com/forum/viewtopic.php?t=6567

-- Ce script permet de gérer le temps de fonctionnement de la pompe de la piscine selon la température de l'eau
-- selon les valeurs définies ci-dessous

--  Température <= 10°C		    : Filtration = 2H	Horaires	14H à 16H
--  Température >10°C et <=12°C : Filtration = 4H	Horaires	10H à 12H	14H à 16H
--  Température >12°C et <=16°C : Filtration = 6H	Horaires	10H à 12H	14H à 16H	17H à 19H
--  Température >16°C et <24°C : Filtration = 8H	Horaires	08H à 10H	12H à 14H	15H à 17H	18H à 20H
--  Température >=24°C et <27°C : Filtration = 12H	Horaires	08H à 20H
--  Température >=27°C et <30°C : Filtration = 20H	Horaires	02H à 22H
--  Température >=30°C           : Filtration = 24H	Horaires	00H à 24H

--  Il possède aussi un mode hors gel une fonction pompe permanente en cas de T° supérieure ou égale à 30
--  Fonction aussi de la vitesse du vent, si vous avez un capteur on peut aussi démarrer la filtration au dessus d'une certaine vitesse

--  Depuis la version 3.0, le script utilise une variable utilisateur (temppiscine) qui est actualisée toutes les heures.
--  Cela évite les arrêts redémarrages de la pompe liés à la lecture temps réel. Une autre option aurait pu être de récupérer
--  la valeur moyenne de la veille comme d'autres le font, mais je préfère toujours avoir une lecture au plus proche.

-- Variables à personaliser attention penser également à créer la variable utilisateur (temppiscine) dans domoticz 
-- comme expliqué dans mon topic

local pompe = 'Pompe Piscine'                    -- Nom de l'interrupteur de la pompe de piscine
local piscine = 'Piscine'                        -- Nom de la sonde de température de l'eau de la piscine
local anemometre = 0                             -- Mettre à 1 si vous voulez utiliser la fonction démarrage de filtration en cas de vent élevé.
local vent = 'Vent / Ressenti'                   -- Nom du capteur de vent à personaliser si vous avez mis anemometre à 1
local vent_vmax = 20    	                 -- Vitesse du vent pour déclencher la filtration
local auto = 'Filtration Automatique'            -- Nom de l'inter virtuel qui permet de passer en mode manuel
local traitement = 'Traitement'                  -- Nom de l'interrupteur des pompes doseuses de traitement (en cours de réflexion : sera intégré sur la V4

-- Récupération de l'heure

time=os.time()
Heure=tonumber(os.date('%H',time))
minute=tonumber(os.date('%M',time))

-- -------------------------------
-- Début des fonctions
-- -------------------------------

-- Fonction Filtration 2H

local function filtration_2H()
  if (Heure >= 14 and Heure < 16 and otherdevices[pompe] == 'Off') then 
    commandArray[pompe]='On'
	print('Programme filtration 2H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .." °C")
  elseif ((Heure < 14 or Heure >= 16) and otherdevices[pompe] == 'On') then 
    commandArray[pompe]='Off'
	print('Programme filtration 2H')
    print('Fin de programme de filtration quotidien')
  end
end

-- Fonction filtation 4H

local function filtration_4H()
  if (Heure >= 10 and Heure < 12 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
    print('Programme filtration 4H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C") 
  elseif ((Heure >= 12 and Heure < 14) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
    print('Programme filtration 4H')
	print("Arrêt pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure >= 14 and Heure < 16 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 4H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif ((Heure < 10 or Heure >= 16) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
        print('Programme filtration 4H')
    print('Fin de programme de filtration quotidien')
  end
end

-- Fonction filtation 6H

local function filtration_6H()
  if (Heure >= 10 and Heure < 12 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
    print('Programme filtration 6H')
	print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure >= 14 and Heure < 16  and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 6H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure >= 17 and Heure < 19 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 6H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (((Heure >= 12 and Heure < 14) or Heure == 16) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
	print('Programme filtration 6H')
	print("Arrêt pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif ((Heure < 10 or Heure >= 19) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
	print('Programme filtration 6H')
    print('Fin de programme de filtration quotidien')
  end
end

-- Fonction filtration 8H

local function filtration_8H()
  if (Heure >= 8 and Heure < 10 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 8H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure >= 12 and Heure < 14 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 8H')
	print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure >= 15 and Heure < 17 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 8H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure >=18 and Heure < 20 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 8H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (((Heure >=10 and Heure <= 11) or Heure == 14 or Heure == 17) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
	print('Programme filtration 8H')
	print("Arrêt pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif ((Heure < 8 or Heure >= 20) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
	print('Programme filtration 8H')
    print('Fin de programme de filtration quotidien')
  end
end

-- Fonction filtration 12H

local function filtration_12H()
  if (Heure >= 8 and Heure < 20 and otherdevices[pompe] == 'Off') then
    commandArray[pompe]='On'
	print('Programme filtration 12H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif ((Heure < 8 or Heure >= 20) and otherdevices[pompe] == 'On') then
    commandArray[pompe]='Off'
	print('Programme filtration 12H')
    print('Fin de programme de filtration quotidien')
  end
end

-- Fonction filtration 20H

local function filtration_20H()
  if Heure >= 2 and Heure < 22 and otherdevices[pompe] == 'Off' then
    commandArray[pompe]='On'
	print('Programme filtration 20H')
    print("Démarrage pompe : Température de l'eau : " .. Temp_piscine .. " °C")
  elseif (Heure < 2 or Heure >= 22) and otherdevices[pompe] == 'On' then
    commandArray[pompe]='Off'
	print('Programme filtration 20H')
	print('Fin de programme de filtration quotidien')
  end
end

-- Fonction filtration permanente

local function filtration_force()
  if otherdevices[pompe] == 'Off' then
    commandArray[pompe]='On'
    print("Filtration 24H/24 : Temperature Piscine trop basse ou trop élévée  " .. Temp_piscine .. " °C")
  end
end

------------------------
-- Fin des Fonctions 
------------------------
 
commandArray = {}

-- Récupération des valeurs de températures piscine et du capteur de vent si le capteur est présent

Temp_piscine, Humidite_piscine = otherdevices_svalues[piscine]:match("([^;]+);([^;]+)")

-- On vérifie si on est pas en mode manuel
-- Permet de pouvoir arrêter ou démarrer la filtration manuellement sans que le script interfère chaque minute
 
if otherdevices[auto] == 'Off' then
     return commandArray
end

-- Si Anémomètre présent alors on récupère la vitesse du vent
-- Si le capteur ne retourne pas de valeur alors on la positonne à 0 pour que test de vitesse mesurée ne plante pas
-- Puis si la vitesse du vent est supérieure à la valeur définie dans les variables alors on démarre la filtration si elle n'est pas déjà démarée

if anemometre == 1 then
  Wbear,Wdir,Wspeed,Wgust,temp_ext,temp_ress = otherdevices_svalues[vent]:match("([^;]+);([^;]+);([^;]+);([^;]+);([^;]+);([^;]+)")

  if Wspeed == nil then
    Wspeed = 0
  else 
    Wspeed = tonumber(Wspeed/2.777778)  -- Ligne à adapter ou à supprimer en fonction de la valeur recue, pour mon capteur, il faut diviser par 2.777778 pour avoir la bonne vitesse en km/h
    Wspeed = tonumber(string.format("%.1f", Wspeed)) 
  end
  if Wspeed > vent_vmax and otherdevices[pompe] == 'Off' then
    commandArray[pompe]='On'
	print("Démarrage pompe : Vitesse du vent élevée : " .. Wspeed .. " Km/h")
	return commandArray
  elseif Wspeed > vent_vmax and otherdevices[pompe] == 'On' then
    return commandArray
  end
end

-- Toutes les heures on actualise la variable utilisateur (temppiscine)

if minute==00 then 
  commandArray['Variable:temppiscine'] = Temp_piscine
end

-- Conversion température piscine depuis la variable en nombre

Temp_piscine_H = tonumber(uservariables['temppiscine'])

-- Conversion températeure piscine temps réel en nombre

Temp_piscine = tonumber(Temp_piscine) 

-- Conditions d'appel aux fonctions de filtration

if (Temp_piscine_H >=3 and Temp_piscine_H <= 10) then
  filtration_2H()
elseif (Temp_piscine_H > 10 and Temp_piscine_H <= 12) then
  filtration_4H()
elseif (Temp_piscine_H > 12 and Temp_piscine_H <= 16) then
  filtration_6H()
elseif (Temp_piscine_H > 16 and Temp_piscine_H < 24) then
  filtration_8H()
elseif (Temp_piscine_H >= 24 and Temp_piscine_H < 27) then
  filtration_12H()
elseif (Temp_piscine_H >= 27 and Temp_piscine_H < 30) then
  filtration_20H()
else
  filtration_force()
end

return commandArray
